/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: ftp_cmd.h,v 1.12 2003/02/24 15:51:12 thepin Exp $
 */


#ifndef _FTP_CMD_H
#define _FTP_CMD_H
#include <ftp.h>
#include <ftp_event_callback.h>

/* function prototypes. */

/* Login (USER/PASS) */
FTP_CMD_PLAIN(ftp_login);

/* TYPE */
FTP_CMD_WRAPPER(ftp_type, int);

/* Uses PORT/PASV depending on mode */
FTP_CMD_WRAPPER(ftp_open_dataconn);

/* QUIT */
FTP_CMD_WRAPPER(ftp_quit_wrapper);
FTP_CMD_WITH_REQ(ftp_quit_with_req);
FTP_CMD_WITH_CB(ftp_quit_with_cb);
FTP_CMD_PLAIN(ftp_quit);

/* CWD */
FTP_CMD_WRAPPER(ftp_cwd_wrapper, const char *);
FTP_CMD_WITH_REQ(ftp_cwd_with_req, const char *);
FTP_CMD_WITH_CB(ftp_cwd_with_cb, const char *);
FTP_CMD_PLAIN(ftp_cwd, const char *);

/* PWD */
FTP_CMD_WRAPPER(ftp_pwd_wrapper);
FTP_CMD_WITH_REQ(ftp_pwd_with_req);
FTP_CMD_WITH_CB(ftp_pwd_with_cb);
FTP_CMD_PLAIN(ftp_pwd);

/* PORT */
FTP_CMD_WRAPPER(ftp_port);

/* PASV */
FTP_CMD_WRAPPER(ftp_passive);

/* RETR */
FTP_CMD_WRAPPER(ftp_retr, struct _list *, char *localfile, int);

/* STOR */
FTP_CMD_WRAPPER(ftp_stor, struct _list *, char *localfile, int);
ftp_request * ftp_mkpath(_ftp_item *ftp, ftp_request *mreq, const char *file);

/* LIST */
FTP_CMD_WRAPPER(ftp_list_wrapper, int, char *);
FTP_CMD_WITH_REQ(ftp_list_with_req, int, char *);
FTP_CMD_WITH_CB(ftp_list_with_cb, int, char *);
FTP_CMD_PLAIN(ftp_list, int, char *);

/* MKDIR */
FTP_CMD_WRAPPER(ftp_mkdir_wrapper, const char *);
FTP_CMD_WITH_REQ(ftp_mkdir_with_req, const char *);
FTP_CMD_WITH_CB(ftp_mkdir_with_cb, const char *);
FTP_CMD_PLAIN(ftp_mkdir, const char *);

/* MOVE */
FTP_CMD_WRAPPER(ftp_move_wrapper, char *file, char *new);

/* DELETE */
FTP_CMD_WRAPPER(ftp_remove_wrapper, char *file);
FTP_CMD_PLAIN(ftp_remove, char *file);

/* REMDIR */
FTP_CB_CMD(ftp_event_remove_dir);
FTP_CB_CMD(ftp_event_remove_dir_listed);
FTP_CMD_WRAPPER(ftp_remdir_wrapper, char *file);
FTP_CMD_PLAIN(ftp_remdir, char *file);

/* NOOPIE NOOP :) */
FTP_CMD_PLAIN(ftp_noop);

#endif
