/* junkie - gui ftp client
 * Copyright (c) 2002-2003, Gunnsteinn Hall, Joel Thomas, Martin Faxer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of the
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
 *  HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: sound_mpg.c,v 1.9 2003/03/03 02:16:34 tgucio Exp $
 */



#include <configure.h>  /* file generated by configure. */

#ifdef HAVE_LIBJAMP
#ifdef HAVE_ESD

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <pthread.h>
#include <esd.h>
/* libjamp */
#include <jamp.h>
/* local header files.  */
#include <defaults.h>
#include <sound.h>
#include <sound_mpg.h>
#include <sound_esd.h>


long frequencies[9] = { 44100, 48000, 32000,
                  22050, 24000, 16000 ,
		  11025 , 12000 , 8000 };



int mpg_fd;
pthread_t thread;
int mpg_playing;
struct sound_mpg_thread smt;



/*
** Preload an mpg file.
** Used for the notification sounds part.
*/
#if 0
int
sound_mpg_preload(char *name, char *path)
{
	/* return the id of the cached sound. */
	return sampleid;
}
#endif


/*
** Init the vorbis support.
** Simply opens up a connection to ESD.
*/
int
sound_mpg_init()
{
	jamp_setup();
	if ((mpg_fd = esd_open_sound(NULL)) < 0) {
		SOUND_DEBUG("sound_vorbis: failed to open ESD sound device\n");
		return 0;
	}
	
	return 1;
}

/*
** shutdown the vorbis support.
** simply closes the connection to esd.
*/
void
sound_mpg_shutdown()
{
	esd_close(mpg_fd);
	mpg_fd = 0;
	return;
}



#if 0
/*
** Play a sound event (for the notification sounds part).
*/
int
sound_vorbis_play_event(struct sound_list *snd)
{
	if (ogg_fd <= 0)
		return 0;

	esd_sample_play(ogg_fd, snd->esd_id);

	return 1;
}
#endif



void
skip_id3_tag(FILE *f)
{
	char buf[4];

	fread(buf, 1, 3, f);
	if (!strcmp(buf, "ID3"))
		fseek(f, 128, SEEK_SET);
	else
		fseek(f, 0, SEEK_SET);
	return;
}

/*
** Play a file encoded by ogg-vorbis.
*/
int
sound_mpg_play_file(char *path, cb callback)
{
	esd_format_t format = 0;
	int bits, channels, rate, mode, func;

	memset(&smt, 0, sizeof(struct sound_mpg_thread));
	if (!(smt.f = fopen(path, "r"))) {
		/* fail to open that file. */
		return 0;		
	}

	if (!jamp_open(smt.f, &smt.jamp)) {
		DEBUG_MSG(DBG_WEIRD, "invalid mp3?");
		return 0;
	}

	fseek(smt.f, 0, SEEK_SET);

	/* setup the format for the ESD. */
	rate = smt.jamp.info.rate;
	bits = ESD_BITS16;
	mode = ESD_STREAM;
	func = ESD_PLAY;
	
	switch (smt.jamp.info.channels) {
		case 1:
			channels = ESD_MONO;
			break;
		default:
			channels = ESD_STEREO;
			break;
	}

	DEBUG_MSG(DBG_INFO, "rate: %d bits: %d mode: %d func: %d channels: %d", rate, bits, mode, func, channels);
	format = bits | channels | mode | func;
	DEBUG_MSG(DBG_INFO, "format is set to 0x%x at rate %d", format, rate);
	smt.sock = esd_play_stream_fallback(format, rate, NULL, NULL);
	DEBUG_MSG(DBG_INFO, "play_stream_fallback_-> done");
	if (smt.sock <= 0) {
		DEBUG_MSG(DBG_ERROR, "failed to connect to esd server");
		return 0;
	}

	smt.operation = MPGPLAYBACK;
	smt.close_after = 1;

	smt.callback = callback;
	
	DEBUG_MSG(DBG_INFO, "going to phtread_create:");
	/* pull off a thread to handle the playback. */
	pthread_create(&thread, NULL, (void *)sound_mpg_update_thread, (void *)&smt);
	DEBUG_MSG(DBG_INFO, "pthread_create = done");
	return 1;
}


int
sound_mpg_stop()
{
	DEBUG_MSG(DBG_INFO, "sound_mpg stop got callled, mpg_playing zeroed");
	mpg_playing = 0;
	return 1;
}

/*
** The thread that handles sending decoding and sending vorbis data to the
** ESD server.
*/
void
sound_mpg_update_thread(void *arg)
{
	struct sound_mpg_thread *mt = NULL;
	char *pnt = NULL;
	int size;
	

	if (!(mt = arg))
		goto end_of_thread;
	
	mpg_playing = 1;
	
	/* mpglib/ESD.  Read tha file and decode and cache on ESD. */
	while (mpg_playing) {
		if (!(pnt = jamp_read(&mt->jamp, &size)))
			break;
			
		write(mt->sock, pnt, size);
		free(pnt);
	}	
	mpg_playing = 0;

	/* clean up. */
	mt->ret = 1;
	if (mt->operation == MPGCACHE) {
		if (esd_confirm_sample_cache(mt->sock) != mt->arg) 
			mt->ret = 0;	/* confirmation refused. */
	}

	/* clean up. */
	fclose(mt->f);
	
	if (mt->close_after)
		close(mt->sock);


end_of_thread:
	/* terminate the threed. */
	if(mt->callback)	mt->callback();
	pthread_exit(NULL);
	
	return;	
}


#endif
#endif
